#![no_std]
#![no_main]
#![feature(async_fn_track_caller, type_alias_impl_trait)]
#![warn(rust_2018_idioms)]
#![forbid(elided_lifetimes_in_paths, unsafe_code)]

mod static_mutex;

use defmt::*;
use defmt_rtt as _;
use embassy_executor::Spawner;
use embassy_stm32::{
	exti::ExtiInput,
	gpio::{Input, Level, Output, Pull, Speed},
	peripherals::{PA0, PA1, PA5, PC13}
};
use embassy_sync::blocking_mutex::raw::ThreadModeRawMutex;
use embassy_time::{Duration, Instant, Timer};
use panic_probe as _;
use static_mutex::StaticMutex;

type Mutex<T> = embassy_sync::mutex::Mutex<ThreadModeRawMutex, T>;
type MutexGuard<'a, T> = embassy_sync::mutex::MutexGuard<'a, ThreadModeRawMutex, T>;

type NucleoLed = Output<'static, PA5>;
type NucleoButton = ExtiInput<'static, PC13>;
type POn = Output<'static, PA0>;
type Tco = ExtiInput<'static, PA1>;

static LED: StaticMutex<NucleoLed> = StaticMutex::new();
static BLINKING: Mutex<bool> = Mutex::new(true);

#[embassy_executor::task]
async fn blinker(interval: Duration) {
	loop {
		if *BLINKING.lock().await {
			LED.lock().await.toggle();
		}
		Timer::after(interval).await;
	}
}

#[embassy_executor::task]
async fn button_handler(mut button: NucleoButton) {
	loop {
		button.wait_for_falling_edge().await;
		let mut blinking = BLINKING.lock().await;
		if *blinking {
			info!("Disable blinking");
			LED.lock().await.set_low();
			*blinking = false;
		} else {
			info!("Enable blinking");
			*blinking = true;
		}
	}
}

#[embassy_executor::task]
async fn query_time(mut pon: POn, mut tco: Tco, interval: Duration) {
	loop {
		pon.set_low();
		info!("Waiting");
		tco.wait_for_falling_edge().await;
		// for _ in 0 .. 10 {
		// 	info!("Receiving");
		// 	let mut bytes = 0u64;
		// 	for _ in 0 .. 60 {
		// 		bytes = (bytes << 1) | (tco.is_low() as u64);
		// 		Timer::after(Duration::from_millis(100)).await;
		// 	}
		// 	info!("DCF77: {:X}", bytes);
		// }
		let start = Instant::now();
		let mut next = start;
		for _ in 0 .. 120 {
			let mut bytes = 0u16;
			for _ in 0 .. 10 {
				bytes = (bytes << 1) | (tco.is_low() as u16);
				next += Duration::from_millis(100);
				Timer::at(next).await;
			}
			println!(
				"{}{}{}{}{}{}{}{}{}{} ({})",
				(bytes >> 9) & 1,
				(bytes >> 8) & 1,
				(bytes >> 7) & 1,
				(bytes >> 6) & 1,
				(bytes >> 5) & 1,
				(bytes >> 4) & 1,
				(bytes >> 3) & 1,
				(bytes >> 2) & 1,
				(bytes >> 1) & 1,
				bytes & 1,
				start.elapsed().as_millis()
			);
		}
		pon.set_high();

		Timer::after(interval).await;
	}
}

#[embassy_executor::main]
async fn main(spawner: Spawner) {
	let p = embassy_stm32::init(Default::default());

	let pon = Output::new(p.PA0, Level::High, Speed::Low);
	let tco = ExtiInput::new(Input::new(p.PA1, Pull::None), p.EXTI1);

	unwrap!(spawner.spawn(query_time(pon, tco, Duration::from_secs(300))));

	LED.init(Output::new(p.PA5, Level::Low, Speed::Low)).await;
	let button = ExtiInput::new(Input::new(p.PC13, Pull::Up), p.EXTI13);

	unwrap!(spawner.spawn(blinker(Duration::from_millis(500))));
	unwrap!(spawner.spawn(button_handler(button)));
}
